import { QueriesObserver, notifyManager } from '@tanstack/query-core';
import { derived, get, readable } from 'svelte/store';
import { useIsRestoring } from './useIsRestoring';
import { useQueryClient } from './useQueryClient';
import { createMemo } from './utils.svelte';
export function createQueries({ queries, ...options }, queryClient) {
    const client = useQueryClient(queryClient);
    let isRestoring = useIsRestoring();
    const queriesStore = $derived(queries);
    const defaultedQueriesStore = createMemo(() => {
        return queries.map((opts) => {
            const defaultedOptions = client.defaultQueryOptions(opts);
            // Make sure the results are already in fetching state before subscribing or updating options
            defaultedOptions._optimisticResults = isRestoring
                ? 'isRestoring'
                : 'optimistic';
            return defaultedOptions;
        });
    });
    const observer = $derived(new QueriesObserver(client, defaultedQueriesStore, options));
    $effect(() => {
        // Do not notify on updates because of changes in the options because
        // these changes should already be reflected in the optimistic result.
        observer.setQueries(defaultedQueriesStore, options, { listeners: false });
    });
    let result = $state();
    $effect(() => {
        const unsubscribe = isRestoring
            ? () => undefined
            : observer.subscribe(notifyManager.batchCalls((v) => {
                result = v;
            }));
        return () => unsubscribe();
    });
    const data = $derived(() => {
        const [rawResult, combineResult, trackResult] = observer.getOptimisticResult(defaultedQueriesStore);
        result = rawResult;
        return combineResult(trackResult());
    });
    return data();
}
